C Program to prepare an input file for epitope mapping programs
C
C Syntax:
C       epiprep [-chains A B ...] < pdbfilein.pdb > pdbfileout.pdb
C
C where A, B, ... are the list of chains to use, over-riding those specified in
C the BIOMOLECULE record
C
C Program appends REMARK 350 records showing mapping from
C PDB ID residue names to sequential names in the output file
C****************************************************************
C Adapted from contacts.f
C RME, Oxford 15/8/12
C****************************************************************

	INTEGER MXATOM
	PARAMETER (MXATOM = 20000)
	INTEGER MXLIST
	PARAMETER (MXLIST = 4*MXATOM)

	INTEGER MXNCS
	PARAMETER (MXNCS = 60)
	INTEGER		NCONT(MXNCS)
	COMMON /CMP/ NCONT
	SAVE /CMP/

	REAL DIST2
	PARAMETER (DIST2 = 4.0*4.0)
	REAL D2

	INTEGER I, J, J1, NJ1, K
	INTEGER	NCONT1(MXNCS), NCONT2(MXNCS)

C Arrays for file input

	CHARACTER*66 ATREC(MXATOM, MXNCS)

C Arrays for atom coordinates

	REAL XYZ(3, MXATOM, MXNCS)

C Arrays for required chain IDs

	INTEGER		MXCH
	PARAMETER	(MXCH=8)
	INTEGER		NCH
	CHARACTER*1     CH(MXCH)

C Arrays for symmetry operators

	INTEGER		NAT(MXNCS), NNCS
	DOUBLE PRECISION MNCS(9,MXNCS)
	REAL		TNCS(3,MXNCS)

	INTEGER		NRES, IXTMP, NATM
	CHARACTER*6	ORES
	CHARACTER*60	RTMP
	DOUBLE PRECISION SX, SY, SZ

C Variables for parsing the command line

        INTEGER COMMAND_ARGUMENT_COUNT, IARG
	CHARACTER*80 ARG

	INTEGER MXSTK
	PARAMETER (MXSTK = 100)
	INTEGER COMPARE, ERRCOD
	INTEGER PIX(MXNCS)
	INTEGER STACK(MXSTK)
        EXTERNAL COMPARE

C Start of the program...

	WRITE(0,*)

C Get the number of command line arguments

        IARG = COMMAND_ARGUMENT_COUNT()
	IF (IARG.EQ.0) THEN
	  NCH = 0
	ELSE
	  CALL GET_COMMAND_ARGUMENT(1, ARG)
	  IF (ARG.NE.'-chains' .OR. IARG.EQ.1) THEN
	    WRITE(0,'(A)') 'Usage: epiprep [-chains A B ...] < in.pdb > out.pdb'
	    STOP
	  ENDIF
	  NCH = IARG - 1
	  IF (NCH.GT.MXCH) THEN
	    WRITE(0,'(A)') 'Too many required chains in argument, change MXCH'
	    STOP
	  ENDIF
          DO I = 1, NCH
	    CALL GET_COMMAND_ARGUMENT(I+1, ARG)
	    IF (ARG(2:).NE.' ') THEN
	      WRITE(0,'(A)') 'Error: chain IDs not single characters'
	      STOP
	    ENDIF
	    CH(I) = ARG(1:1)
	  ENDDO
	ENDIF

C We are specifying operators, so read a single PDB model and then the operators
C in the form produced by Robert's little impose program

	CALL READPDB(NAT(1), NNCS, NCH, MXATOM, MXNCS, MXCH, ATREC, XYZ, MNCS, TNCS, CH)

C Return to here after detecting that VP3 needs repositioning first and recalculate everything

11	WRITE(0,*)

C Generate all the NCS related copies of the first model

	  DO J = 2, NNCS
	    DO I = 1, NAT(1)
	      ATREC(I, J) = ATREC(I, 1)
	    ENDDO
	    NAT(J) = NAT(1)
	  ENDDO

C Generate all the strict-NCS related copies of the molecule

 	CALL GENNCS(NNCS, NAT(1), MXATOM, MXNCS, XYZ, MNCS, TNCS)

C Calculate the centre of the complete particle

	SX = 0.0
	SY = 0.0
	SZ = 0.0
	DO J = 1, NNCS
	  SX = SX + XYZ(1, 1, J)
	  SY = SY + XYZ(2, 1, J)
	  SZ = SZ + XYZ(3, 1, J)
	ENDDO
	SX = SX / NNCS
	SY = SY / NNCS
	SZ = SZ / NNCS

C Now search for contacts between NCS molecule 1 and all other molecules keeping
C a count in the NCONT array

	DO J = 1, NNCS
	  NCONT(J) = 0
	  PIX(J) = J
	ENDDO
	DO I = 1, NAT(1)
	  DO J = 2, NNCS
	    DO K = 1, NAT(J)
	      D2 = (XYZ(1,I,1)-XYZ(1,K,J))**2
     &           + (XYZ(2,I,1)-XYZ(2,K,J))**2
     &           + (XYZ(3,I,1)-XYZ(3,K,J))**2
	      IF (D2.LE.DIST2) NCONT(J) = NCONT(J) + 1
	    ENDDO
	  ENDDO
	ENDDO

C Sort the contacts to give the most important NCS contacts

	CALL QSORTP(PIX, NNCS, STACK, MXSTK, COMPARE, ERRCOD)

C Now test whether there is a clear difference between the counts for the third
C and fourth NCS operator. If not then we have to move molecule VP3 first.

	IF (1.0*NCONT(PIX(3)) .LT. 1.1*NCONT(PIX(4))) THEN
	  WRITE(0,'(A)') 'Calculating which VP3 operator gives single protomer'

C Only two operators applied to VP3 (one the identity) will generate contacts
C between VP3 and both VP1 and VP2. Find the other operator, apply it in situ
C to the original copy of VP3 and re-run the program.

	  DO J = 1, NNCS
	    NCONT1(J) = 0
	    NCONT2(J) = 0
	  ENDDO
	  DO I = 1, NAT(1)
	    DO J = 1, NNCS
	      DO K = 1, NAT(J)
	        D2 = (XYZ(1,I,1)-XYZ(1,K,J))**2
     &             + (XYZ(2,I,1)-XYZ(2,K,J))**2
     &             + (XYZ(3,I,1)-XYZ(3,K,J))**2
	        IF (D2.LE.DIST2) THEN
                  IF (ATREC(I,1)(22:22).EQ.'1'
     &              .AND. ATREC(K,J)(22:22).EQ.'3') THEN
	            NCONT1(J) = NCONT1(J) + 1
		  ENDIF
                  IF (ATREC(I,1)(22:22).EQ.'2'
     &              .AND. ATREC(K,J)(22:22).EQ.'3') THEN
	            NCONT2(J) = NCONT2(J) + 1
		  ENDIF
	        ENDIF
	      ENDDO
	    ENDDO
	  ENDDO
	  J1 = 2
	  NJ1 = NCONT1(2)*NCONT2(2)
	  DO J = 3, NNCS
	    IF (NCONT1(J)*NCONT2(J) .GT. NJ1) THEN
	      J1 = J
	      NJ1 = NCONT1(J)*NCONT2(J)
	    ENDIF
	  ENDDO
	  WRITE(0,'(A,I2,A,I4,A,I4,A)') 'Applying operator', J1,
     &    ' to VP3 generates',NCONT1(J1), ' and', NCONT2(J1),
     &    ' contacts with VP1 and VP2'

C Copy across selected coordinates for VP3 only

	  DO I = 1, NAT(1)
            IF (ATREC(I,1)(22:22).EQ.'3') THEN
              XYZ(1,I,1) = XYZ(1,I,J1)
              XYZ(2,I,1) = XYZ(2,I,J1)
              XYZ(3,I,1) = XYZ(3,I,J1)
	    ENDIF
	  ENDDO

	  WRITE(0,'(A)') 'Recalculating with repositioned VP3'
	  GOTO 11
	ENDIF

C Write out the recentering and remapping of the residue numbers as a header

	WRITE(0,'(A)') 'Centering particle on the origin'
	WRITE(6,'(A,3F8.3,A)') 'REMARK Centre of capsid was ', SX, SY, SZ,
     &                         ' and has been shifted to the origin here'

	WRITE(0,'(A)') 'Writing REMARK record for residue renumbering'
	NRES = 0
	IXTMP = 0
	ORES = '%%%%%%'
	DO I = 1, NAT(1)
	  IF (ORES.NE.ATREC(I,1)(22:27)) THEN
	    ORES = ATREC(I,1)(22:27)
	    RTMP(IXTMP+1:IXTMP+6) = ORES
	    IXTMP = IXTMP + 6
	    IF (IXTMP.EQ.60) THEN
	      WRITE(6,'(A,I4,A,A)') 'REMARK RMAP ', NRES+1, '    ',RTMP
	      NRES = NRES + 10
	      IXTMP = 0
	    ENDIF
	  ENDIF
	ENDDO
	IF (IXTMP.GT.0) THEN
	  WRITE(6,'(A,I4,A,A)') 'REMARK RMAP ', NRES+1, '    ',RTMP(:IXTMP)
	ENDIF

C Write out the PDB file of the originial protomer and the three major
C surrounding protomers

	WRITE(0,*)
	NATM = 0
	DO J = 1, 4
	  IF (J.EQ.1) THEN
	    J1 = 1
	    WRITE(0,'(A,I2,A)') 'Writing ATOM records for NCS operator ', J1,
     &                      ' (identity operator)'
	  ELSE
	    J1 = PIX(J-1)
	    WRITE(0,'(A,I2,A,I4,A)') 'Writing ATOM records for NCS operator ', J1,
     &                      ' (', NCONT(J1),' contacts)'
	  ENDIF
	  NRES = 0
	  ORES = '%%%%%%'
	  DO I = 1, NAT(J1)
	    NATM = NATM + 1
	    IF (ORES.NE.ATREC(I,J1)(22:27)) THEN
	      ORES = ATREC(I,J1)(22:27)
	      NRES = NRES + 1
	    ENDIF
	    WRITE(6,'(A,I5,A,I1,I4,A,A,3F8.3,A)')
     &      ATREC(I,J1)(:6), NATM,
     &      ATREC(I,J1)(12:21), J, NRES, ' ',
     &      ATREC(I,J1)(28:30),
     &      XYZ(1,I,J1)-SX, XYZ(2,I,J1)-SY, XYZ(3,I,J1)-SZ,
     &      ATREC(I,J1)(55:66)
	  ENDDO
	ENDDO
	J1 = PIX(4)
	WRITE(0,'(A,I2,A,I4,A)') 'Next NCS operator (', J1,
     &                  ') makes', NCONT(J1),' contacts'
	J1 = PIX(5)
	WRITE(0,'(A,I2,A,I4,A)') 'Next NCS operator (', J1,
     &                  ') makes', NCONT(J1),' contacts'
	J1 = PIX(6)
	WRITE(0,'(A,I2,A,I4,A)') 'Next NCS operator (', J1,
     &                  ') makes', NCONT(J1),' contacts'
	WRITE(0,*)

	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C Read in atomic coordinates from stdin
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE READPDB(NAT, NNCS, NCH, MXATOM, MXNCS, MXCH, ATREC, XYZ,
     &                     MNCS, TNCS, CH)
	INTEGER		NAT, NNCS, NCH, MXATOM, MXNCS, MXCH
	CHARACTER*(*)	ATREC(MXATOM)
	REAL		XYZ(3, MXATOM)
	DOUBLE PRECISION MNCS(9,MXNCS)
	REAL		TNCS(3,MXNCS)
	CHARACTER*(*)	CH(MXCH)

	CHARACTER*80	LINE
	INTEGER		ILEN

	INTEGER		IIN
	INTEGER		NIGN, NIX, ICH
	CHARACTER*1	CCH
	CHARACTER*24	FMAT

C Read the file ignoring hydrogen atoms

	NAT = 0
	NIGN = 0

	NNCS = 0
	IIN = 5

10	READ(IIN,'(A)',END=19) LINE
	DO ILEN = 80, 1, -1
	  IF (LINE(ILEN:ILEN).NE.' ') GOTO 11
	ENDDO
11      IF (ILEN.LT.4) GOTO 10
	IF (LINE(:10).EQ.'REMARK 350') THEN
	  IF (LINE(12:24).EQ.'BIOMOLECULE: ') THEN
	    IF (LINE(25:25).NE.'1') THEN
	      WRITE(0,'(A)') 'Records for other than biomolecule 1'
	      STOP
	    ENDIF
	  ELSE IF (LINE(12:41).EQ.'APPLY THE FOLLOWING TO CHAINS:') THEN
	    IF (NCH.EQ.0) THEN
	      NIX = 43
12	      IF (LINE(NIX:NIX).NE.' ') THEN
	        IF (NCH.EQ.MXCH) THEN
	          WRITE(0,'(A)') 'Too many required chains in file, change MXCH'
	          STOP
	        ENDIF
	        NCH = NCH + 1
	        CH(NCH) = LINE(NIX:NIX)
	        NIX = NIX + 3
	        GOTO 12
              ENDIF
	    ENDIF
	  ELSE IF (LINE(14:19).EQ.'BIOMT1') THEN
	    IF (NNCS.EQ.MXNCS) THEN
	      WRITE(0,'(A)') 'Too many NCS operators in file, change MXNCS'
	      STOP
	    ENDIF
	    NNCS = NNCS + 1
	    READ(LINE(24:68), '(3D10.6,5X,F10.5)') MNCS(1,NNCS), MNCS(2,NNCS),
     1                                             MNCS(3,NNCS), TNCS(1,NNCS)
	  ELSE IF (LINE(14:19).EQ.'BIOMT2') THEN
	    READ(LINE(24:68), '(3D10.6,5X,F10.5)') MNCS(4,NNCS), MNCS(5,NNCS),
     1                                             MNCS(6,NNCS), TNCS(2,NNCS)
	  ELSE IF (LINE(14:19).EQ.'BIOMT3') THEN
	    READ(LINE(24:68), '(3D10.6,5X,F10.5)') MNCS(7,NNCS), MNCS(8,NNCS),
     1                                             MNCS(9,NNCS), TNCS(3,NNCS)
	  ENDIF

C Process searching for ATOM records (not HETATM any more)

	ELSE IF (LINE(:4).EQ.'ATOM') THEN
	  IF (LINE(13:14).EQ.' H'.OR.LINE(13:13).EQ.'H') GOTO 10
	  CCH = LINE(22:22)
	  DO ICH = 1, NCH
	    IF (CCH.EQ.CH(ICH)) GOTO 18
	  ENDDO
	  NIGN = NIGN + 1
	  GOTO 10
18	  IF (NAT.EQ.MXATOM) THEN
	    WRITE(0,'(A)') 'Too many atoms in PDB file, change MXATOM'
	    STOP
	  ENDIF
	  NAT = NAT + 1
	  ATREC(NAT) = LINE
	  READ(LINE(31:54),'(3F8.3)') XYZ(1,NAT), XYZ(2,NAT), XYZ(3,NAT)
	ENDIF
	GOTO 10

19	FMAT = '(A, 12('' "'',A1,''"''))'
	WRITE(FMAT(4:6), '(I3)') NCH
	WRITE(0,FMAT) 'Selected chains:', (CH(ICH),ICH=1,NCH)
	WRITE(0,'(A,I6)') 'Number of NCS operators read from header:  ', NNCS
	WRITE(0,'(A,I6)') 'Number of ATOM records in selected chains: ', NAT
	WRITE(0,'(A,I6)') 'Number of ATOM records in other chains:    ', NIGN
	RETURN

	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C Generate NCS related copies of the molecule
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE GENNCS(NNCS, NAT, MXATOM, MXNCS, XYZ, MNCS, TNCS)
	INTEGER NNCS, NAT, MXATOM, MXNCS
	REAL	XYZ(3, MXATOM, MXNCS)
	DOUBLE PRECISION MNCS(9, MXNCS)
	REAL	TNCS(3, MXNCS)

	INTEGER I, J
	REAL	V(3)

C Here is where NCS related copies of the molecule are generated. NCS operators
C should be specified as a transformation in orthogonal coordinates from the
C copy of the molecule read in.

	DO J = 2, NNCS
	  DO I = 1, NAT
	    CALL VECMUL(V, MNCS(1,J), XYZ(1,I,1))
	    CALL VECADD(XYZ(1,I,J), V, TNCS(1,J))
	  ENDDO
	ENDDO

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C Multiply one 3x3 matrix by another in the sense M1 = M2 M3
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE MATMUL(M1, M2, M3)

	DOUBLE PRECISION M1(9), M2(9), M3(9)

	INTEGER I
	DOUBLE PRECISION T(9)

	T(1) = M2(1) * M3(1) + M2(2) * M3(4) + M2(3) * M3(7)
	T(2) = M2(1) * M3(2) + M2(2) * M3(5) + M2(3) * M3(8)
	T(3) = M2(1) * M3(3) + M2(2) * M3(6) + M2(3) * M3(9)
	T(4) = M2(4) * M3(1) + M2(5) * M3(4) + M2(6) * M3(7)
	T(5) = M2(4) * M3(2) + M2(5) * M3(5) + M2(6) * M3(8)
	T(6) = M2(4) * M3(3) + M2(5) * M3(6) + M2(6) * M3(9)
	T(7) = M2(7) * M3(1) + M2(8) * M3(4) + M2(9) * M3(7)
	T(8) = M2(7) * M3(2) + M2(8) * M3(5) + M2(9) * M3(8)
	T(9) = M2(7) * M3(3) + M2(8) * M3(6) + M2(9) * M3(9)

C Copy the final matrix to the result

	DO I = 1, 9
	  M1(I) = T(I)
	ENDDO

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C Pre-multiply a 3D vector by a 3x3 matrix in the sense V1 = M2 V3
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE VECMUL(V1, M2, V3)

	REAL V1(3), V3(3)
	DOUBLE PRECISION M2(9)

	INTEGER I
	REAL T(3)

	T(1) = M2(1) * V3(1) + M2(2) * V3(2) + M2(3) * V3(3)
	T(2) = M2(4) * V3(1) + M2(5) * V3(2) + M2(6) * V3(3)
	T(3) = M2(7) * V3(1) + M2(8) * V3(2) + M2(9) * V3(3)

C Copy the final vector to the result

	DO I = 1, 3
	  V1(I) = T(I)
	ENDDO

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C Add two 3D vectors together in the sense V1 = V2 + V3
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE VECADD(V1, V2, V3)

	REAL V1(3), V2(3), V3(3)

	V1(1) = V2(1) + V3(1)
	V1(2) = V2(2) + V3(2)
	V1(3) = V2(3) + V3(3)

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C Set one 3D vector to be another one in the sense V1 = V2
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	SUBROUTINE VECCOPY(V1, V2)

	REAL V1(3), V2(3)

	V1(1) = V2(1)
	V1(2) = V2(2)
	V1(3) = V2(3)

	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

	INTEGER FUNCTION COMPARE(I1, I2)
	INTEGER I1, I2

	INTEGER MXNCS
	PARAMETER (MXNCS = 60)
	INTEGER		NCONT(MXNCS)
	COMMON /CMP/ NCONT
	SAVE /CMP/

	COMPARE = NCONT(I2) - NCONT(I1)
	IF (COMPARE.LT.0) COMPARE = -1
	IF (COMPARE.GT.0) COMPARE = 1
	RETURN
	END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C QSORTP: QSORT routine for sorting vector lengths
C
C RME Oct 99 - taken from MolScript
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      SUBROUTINE QSORTP(PARRAY, TOT, STACK, MAXSTK, COMPAR, ERRCOD)

      INTEGER TOT, MAXSTK, COMPAR, ERRCOD
      INTEGER PARRAY (TOT), STACK (MAXSTK)
      EXTERNAL COMPAR

      INTEGER S, L, R, I, J, X, PSWAP

C Just skip if nothing to sort

      ERRCOD = 0
      IF (TOT .LE. 1) RETURN

C Initialize stack

      S = 2
      IF (S .GT. MAXSTK) GOTO 900
      STACK (S - 1) = 1
      STACK (S)     = TOT

C Simulated repeat-until loop 1; pop stack until empty

100   L = STACK (S - 1)
      R = STACK (S)
      S = S - 2

C Simulated repeat-until loop 2; split interval until nothing left

200     I = L
        J = R

C Choose (arbitrarily) value in middle of interval as comparand

        X = (L + R) / 2

C Simulated repeat-until loop 3; partition until nothing left

300       IF (COMPAR (PARRAY (I), PARRAY (X)) .LT. 0) THEN
            I = I + 1
            GOTO 300
          END IF
310       IF (COMPAR (PARRAY (X), PARRAY (J)) .LT. 0) THEN
            J = J - 1
            GOTO 310
          END IF

C Swap array values if on correct side of each other in interval;
C beware of what happens to the pointer to the comparand slot

          IF (I .LE. J) THEN
            PSWAP      = PARRAY (I)
            PARRAY (I) = PARRAY (J)
            PARRAY (J) = PSWAP
            IF (X .EQ. I) THEN
              X = J
            ELSE IF (X .EQ. J) THEN
              X = I
            END IF
            I = I + 1
            J = J - 1
          END IF

C End of simulated repeat-until loop 3

          IF (I .LE. J) GOTO 300

C Recursion; process shorter interval first, and push interval to stack

        IF (J - L .LT. R - I) THEN
          IF (I .LT. R) THEN
            S = S + 2
            IF (S .GT. MAXSTK) GOTO 900
            STACK (S - 1) = I
            STACK (S)     = R
          END IF
          R = J

        ELSE
          IF (L .LT. J) THEN
            S = S + 2
            IF (S .GT. MAXSTK) GOTO 900
            STACK (S - 1) = L
            STACK (S)     = J
          END IF
          L = I
        END IF

C End of simulated repeat-until loop 2

        IF (L .LT. R) GOTO 200

C End of simulated repeat-until loop 1

      IF (S .GT. 0) GOTO 100

      RETURN

900   ERRCOD = 1

      RETURN
      END

